@@ -39,6 +39,7 @@
 #include "GOwnPtr.h"
 #include "LayoutTestController.h"
 #include "PixelDumpSupport.h"
+#include "TextInputController.h"
 #include "WebCoreSupport/DumpRenderTreeSupportGtk.h"
 #include "WorkQueue.h"
 #include "WorkQueueItem.h"
@@ -104,6 +105,11 @@ static bool shouldOpenWebInspector(const string& pathOrURL)
     return pathOrURL.find("inspector/") != string::npos;
 }
 
+static bool shouldDumpAsText(const string& pathOrURL)
+{
+    return pathOrURL.find("dumpAsText/") != string::npos;
+}
+
 static bool shouldEnableDeveloperExtras(const string& pathOrURL)
 {
     return true;
@@ -231,6 +237,13 @@ static void initializeFonts(const char* testURL = 0)
     if (!FcConfigAppFontAddFile(config, reinterpret_cast<FcChar8*>(ahemFontFilename.get())))
         g_error("Could not load font at %s!", ahemFontFilename.get()); 
 
+    for (int i = 1; i <= 9; i++) {
+        GOwnPtr<gchar> fontFilename(g_strdup_printf("WebKitWeightWatcher%i00.ttf", i));
+        GOwnPtr<gchar> fontPath(g_build_filename(FONTS_CONF_DIR, "..", "..", "fonts", fontFilename.get(), NULL));
+        if (!FcConfigAppFontAddFile(config, reinterpret_cast<FcChar8*>(fontPath.get())))
+            g_error("Could not load font at %s!", fontPath.get()); 
+    }
+
     // A font with no valid Fontconfig encoding to test https://bugs.webkit.org/show_bug.cgi?id=47452
     GOwnPtr<gchar> fontWithNoValidEncodingFilename(g_build_filename(FONTS_CONF_DIR, "FontWithNoValidEncoding.fon", NULL));
     if (!FcConfigAppFontAddFile(config, reinterpret_cast<FcChar8*>(fontWithNoValidEncodingFilename.get())))
@@ -408,7 +421,7 @@ static void resetDefaultsToConsistentValues()
                  "fantasy-font-family", "fantasy",
                  "default-font-size", 16,
                  "default-monospace-font-size", 13,
-                 "minimum-font-size", 1,
+                 "minimum-font-size", 0,
                  "enable-caret-browsing", FALSE,
                  "enable-page-cache", FALSE,
                  "auto-resize-window", TRUE,
@@ -419,6 +432,7 @@ static void resetDefaultsToConsistentValues()
                  "enable-fullscreen", TRUE,
                  NULL);
     webkit_web_view_set_settings(webView, settings);
+    webkit_set_cache_model(WEBKIT_CACHE_MODEL_DOCUMENT_BROWSER);
 
     DumpRenderTreeSupportGtk::clearMainFrameName(mainFrame);
 
@@ -426,13 +440,13 @@ static void resetDefaultsToConsistentValues()
     g_object_set(G_OBJECT(inspector), "javascript-profiling-enabled", FALSE, NULL);
 
     webkit_web_view_set_zoom_level(webView, 1.0);
+    DumpRenderTreeSupportGtk::setMinimumTimerInterval(webView, DumpRenderTreeSupportGtk::defaultMinimumTimerInterval());
 
     DumpRenderTreeSupportGtk::resetOriginAccessWhiteLists();
 
     WebKitWebBackForwardList* list = webkit_web_view_get_back_forward_list(webView);
     webkit_web_back_forward_list_clear(list);
 
-#ifdef HAVE_LIBSOUP_2_29_90
     SoupSession* session = webkit_get_default_session();
     SoupCookieJar* jar = reinterpret_cast<SoupCookieJar*>(soup_session_get_feature(session, SOUP_TYPE_COOKIE_JAR));
 
@@ -440,11 +454,17 @@ static void resetDefaultsToConsistentValues()
     // HTTP. Should we initialize it earlier, perhaps?
     if (jar)
         g_object_set(G_OBJECT(jar), SOUP_COOKIE_JAR_ACCEPT_POLICY, SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY, NULL);
-#endif
 
     setlocale(LC_ALL, "");
 
     DumpRenderTreeSupportGtk::setLinksIncludedInFocusChain(true);
+    webkit_icon_database_set_path(webkit_get_icon_database(), 0);
+    DumpRenderTreeSupportGtk::setSelectTrailingWhitespaceEnabled(false);
+
+    if (axController)
+        axController->resetToConsistentState();
+
+    DumpRenderTreeSupportGtk::clearOpener(mainFrame);
 }
 
 static bool useLongRunningServerMode(int argc, char *argv[])
@@ -500,7 +520,7 @@ void dump()
         gchar* responseMimeType = webkit_web_frame_get_response_mime_type(mainFrame);
 
         if (g_str_equal(responseMimeType, "text/plain")) {
-            gLayoutTestController->setDumpAsText(true);        
+            gLayoutTestController->setDumpAsText(true);
             gLayoutTestController->setGeneratePixelResults(false);
         }
         g_free(responseMimeType);
@@ -626,6 +646,10 @@ static void runTest(const string& testPathOrURL)
         gLayoutTestController->setDeveloperExtrasEnabled(true);
         if (shouldOpenWebInspector(testURL))
             gLayoutTestController->showWebInspector();
+        if (shouldDumpAsText(testURL)) {
+            gLayoutTestController->setDumpAsText(true);
+            gLayoutTestController->setGeneratePixelResults(false);
+        }
     }
 
     WorkQueue::shared()->clear();
@@ -738,6 +762,11 @@ static void webViewLoadFinished(WebKitWebView* view, WebKitWebFrame* frame, void
         dump();
 }
 
+static gboolean webViewLoadError(WebKitWebView*, WebKitWebFrame*, gchar*, gpointer, gpointer)
+{
+    return TRUE; // Return true here to disable the default error page.
+}
+
 static void webViewDocumentLoadFinished(WebKitWebView* view, WebKitWebFrame* frame, void*)
 {
     if (!done && gLayoutTestController->dumpFrameLoadCallbacks()) {
@@ -781,6 +810,11 @@ static void webViewWindowObjectCleared(WebKitWebView* view, WebKitWebFrame* fram
     JSValueRef eventSender = makeEventSender(context, !webkit_web_frame_get_parent(frame));
     JSObjectSetProperty(context, windowObject, eventSenderStr, eventSender, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete, 0);
     JSStringRelease(eventSenderStr);
+
+    JSStringRef textInputControllerStr = JSStringCreateWithUTF8CString("textInputController");
+    JSValueRef textInputController = makeTextInputController(context);
+    JSObjectSetProperty(context, windowObject, textInputControllerStr, textInputController, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete, 0);
+    JSStringRelease(textInputControllerStr);
 }
 
 static gboolean webViewConsoleMessage(WebKitWebView* view, const gchar* message, unsigned int line, const gchar* sourceId, gpointer data)
@@ -990,6 +1024,17 @@ static void frameCreatedCallback(WebKitWebView* webView, WebKitWebFrame* webFram
     g_signal_connect(webFrame, "notify::load-status", G_CALLBACK(webFrameLoadStatusNotified), NULL);
 }
 
+static void willSendRequestCallback(WebKitWebView* webView, WebKitWebFrame*, WebKitWebResource*, WebKitNetworkRequest* request, WebKitNetworkResponse*)
+{
+    SoupMessage* soupMessage = webkit_network_request_get_message(request);
+
+    if (soupMessage) {
+        const set<string>& clearHeaders = gLayoutTestController->willSendRequestClearHeaders();
+        for (set<string>::const_iterator header = clearHeaders.begin(); header != clearHeaders.end(); ++header)
+            soup_message_headers_remove(soupMessage->request_headers, header->c_str());
+    }
+}
+
 static WebKitWebView* createWebView()
 {
     WebKitWebView* view = WEBKIT_WEB_VIEW(webkit_web_view_new());
@@ -999,6 +1044,7 @@ static WebKitWebView* createWebView()
     g_object_connect(G_OBJECT(view),
                      "signal::load-started", webViewLoadStarted, 0,
                      "signal::load-finished", webViewLoadFinished, 0,
+                     "signal::load-error", webViewLoadError, 0,
                      "signal::window-object-cleared", webViewWindowObjectCleared, 0,
                      "signal::console-message", webViewConsoleMessage, 0,
                      "signal::script-alert", webViewScriptAlert, 0,
@@ -1017,6 +1063,7 @@ static WebKitWebView* createWebView()
                      "signal::drag-end", dragEndCallback, 0,
                      "signal::drag-failed", dragFailedCallback, 0,
                      "signal::frame-created", frameCreatedCallback, 0,
+                     "signal::resource-request-starting", willSendRequestCallback, 0,
 
                      NULL);
     connectEditingCallbacks(view);
