@@ -3,7 +3,7 @@
  * Copyright (C) 2007 Eric Seidel <eric@webkit.org>
  * Copyright (C) 2008 Nuanti Ltd.
  * Copyright (C) 2009 Jan Michael Alonzo <jmalonzo@gmail.com>
- * Copyright (C) 2009 Collabora Ltd.
+ * Copyright (C) 2009,2011 Collabora Ltd.
  * Copyright (C) 2010 Joone Hur <joone@kldp.org>
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,6 @@
 #include <wtf/gobject/GOwnPtr.h>
 
 extern "C" {
-void webkit_application_cache_set_maximum_size(unsigned long long size);
 void webkit_web_inspector_execute_script(WebKitWebInspector* inspector, long callId, const gchar* script);
 }
 
@@ -230,7 +229,6 @@ void LayoutTestController::setAcceptsEditing(bool acceptsEditing)
 
 void LayoutTestController::setAlwaysAcceptCookies(bool alwaysAcceptCookies)
 {
-#ifdef HAVE_LIBSOUP_2_29_90
     SoupSession* session = webkit_get_default_session();
     SoupCookieJar* jar = reinterpret_cast<SoupCookieJar*>(soup_session_get_feature(session, SOUP_TYPE_COOKIE_JAR));
 
@@ -251,7 +249,6 @@ void LayoutTestController::setAlwaysAcceptCookies(bool alwaysAcceptCookies)
         policy = SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY;
 
     g_object_set(G_OBJECT(jar), SOUP_COOKIE_JAR_ACCEPT_POLICY, policy, NULL);
-#endif
 }
 
 void LayoutTestController::setCustomPolicyDelegate(bool setDelegate, bool permissive)
@@ -335,6 +332,11 @@ void LayoutTestController::setUserStyleSheetLocation(JSStringRef path)
         setUserStyleSheetEnabled(true);
 }
 
+void LayoutTestController::setValueForUser(JSContextRef context, JSValueRef nodeObject, JSStringRef value)
+{
+    DumpRenderTreeSupportGtk::setValueForUser(context, nodeObject, value);
+}
+
 void LayoutTestController::setViewModeMediaFeature(JSStringRef mode)
 {
     WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
@@ -456,6 +458,11 @@ void LayoutTestController::setAuthorAndUserStylesEnabled(bool flag)
     // FIXME: implement
 }
 
+void LayoutTestController::setAutofilled(JSContextRef context, JSValueRef nodeObject, bool isAutofilled)
+{
+    DumpRenderTreeSupportGtk::setAutofilled(context, nodeObject, isAutofilled);
+}
+
 void LayoutTestController::disableImageLoading()
 {
     // FIXME: Implement for testing fix for https://bugs.webkit.org/show_bug.cgi?id=27896
@@ -486,15 +493,26 @@ void LayoutTestController::setGeolocationPermission(bool allow)
     setGeolocationPermissionCommon(allow);
 }
 
+int LayoutTestController::numberOfPendingGeolocationPermissionRequests()
+{
+    // FIXME: Implement for Geolocation layout tests.
+    return -1;
+}
+
 void LayoutTestController::addMockSpeechInputResult(JSStringRef result, double confidence, JSStringRef language)
 {
     // FIXME: Implement for speech input layout tests.
     // See https://bugs.webkit.org/show_bug.cgi?id=39485.
 }
 
-void LayoutTestController::setIconDatabaseEnabled(bool flag)
+void LayoutTestController::setIconDatabaseEnabled(bool enabled)
 {
-    // FIXME: implement
+    WebKitIconDatabase* database = webkit_get_icon_database();
+    if (enabled) {
+        GOwnPtr<gchar> iconDatabasePath(g_build_filename(g_get_tmp_dir(), "DumpRenderTree", "icondatabase", NULL));
+        webkit_icon_database_set_path(database, iconDatabasePath.get());
+    } else
+        webkit_icon_database_set_path(database, 0);
 }
 
 void LayoutTestController::setJavaScriptProfilingEnabled(bool flag)
@@ -510,7 +528,7 @@ void LayoutTestController::setJavaScriptProfilingEnabled(bool flag)
 
 void LayoutTestController::setSelectTrailingWhitespaceEnabled(bool flag)
 {
-    // FIXME: implement
+    DumpRenderTreeSupportGtk::setSelectTrailingWhitespaceEnabled(flag);
 }
 
 void LayoutTestController::setPopupBlockingEnabled(bool flag)
@@ -546,10 +564,42 @@ void LayoutTestController::execCommand(JSStringRef name, JSStringRef value)
     g_free(cValue);
 }
 
-bool LayoutTestController::findString(JSContextRef /* context */, JSStringRef /* target */, JSObjectRef /* optionsArray */)
+bool LayoutTestController::findString(JSContextRef context, JSStringRef target, JSObjectRef optionsArray)
 {
-    // FIXME: Implement
-    return false;
+    WebKitFindOptions findOptions = 0;
+    WebKitWebView* webView = webkit_web_frame_get_web_view(mainFrame);
+    ASSERT(webView);
+
+    JSRetainPtr<JSStringRef> lengthPropertyName(Adopt, JSStringCreateWithUTF8CString("length"));
+    JSValueRef lengthValue = JSObjectGetProperty(context, optionsArray, lengthPropertyName.get(), 0); 
+    if (!JSValueIsNumber(context, lengthValue))
+        return false;
+
+    GOwnPtr<gchar> targetString(JSStringCopyUTF8CString(target));
+
+    size_t length = static_cast<size_t>(JSValueToNumber(context, lengthValue, 0));
+    for (size_t i = 0; i < length; ++i) {
+        JSValueRef value = JSObjectGetPropertyAtIndex(context, optionsArray, i, 0); 
+        if (!JSValueIsString(context, value))
+            continue;
+    
+        JSRetainPtr<JSStringRef> optionName(Adopt, JSValueToStringCopy(context, value, 0));
+
+        if (JSStringIsEqualToUTF8CString(optionName.get(), "CaseInsensitive"))
+            findOptions |= WebKit::WebFindOptionsCaseInsensitive;
+        else if (JSStringIsEqualToUTF8CString(optionName.get(), "AtWordStarts"))
+            findOptions |= WebKit::WebFindOptionsAtWordStarts;
+        else if (JSStringIsEqualToUTF8CString(optionName.get(), "TreatMedialCapitalAsWordStart"))
+            findOptions |= WebKit::WebFindOptionsTreatMedialCapitalAsWordStart;
+        else if (JSStringIsEqualToUTF8CString(optionName.get(), "Backwards"))
+            findOptions |= WebKit::WebFindOptionsBackwards;
+        else if (JSStringIsEqualToUTF8CString(optionName.get(), "WrapAround"))
+            findOptions |= WebKit::WebFindOptionsWrapAround;
+        else if (JSStringIsEqualToUTF8CString(optionName.get(), "StartInSelection"))
+            findOptions |= WebKit::WebFindOptionsStartInSelection;
+    }   
+
+    return DumpRenderTreeSupportGtk::findString(webView, targetString.get(), findOptions); 
 }
 
 bool LayoutTestController::isCommandEnabled(JSStringRef name)
@@ -565,10 +615,20 @@ bool LayoutTestController::isCommandEnabled(JSStringRef name)
 
 void LayoutTestController::setCacheModel(int cacheModel)
 {
-    if (!cacheModel) // WebCacheModelDocumentViewer
-        webkit_set_cache_model(WEBKIT_CACHE_MODEL_DOCUMENT_VIEWER); 
-    else 
-        webkit_set_cache_model(WEBKIT_CACHE_MODEL_WEB_BROWSER); 
+    // These constants are derived from the Mac cache model enum in Source/WebKit/mac/WebView/WebPreferences.h.
+    switch (cacheModel) {
+    case 0:
+        webkit_set_cache_model(WEBKIT_CACHE_MODEL_DOCUMENT_VIEWER);
+        break;
+    case 1:
+        webkit_set_cache_model(WEBKIT_CACHE_MODEL_DOCUMENT_BROWSER);
+        break;
+    case 3:
+        webkit_set_cache_model(WEBKIT_CACHE_MODEL_DOCUMENT_BROWSER);
+        break;
+    default:
+        ASSERT_NOT_REACHED();
+    }
 }
 
 void LayoutTestController::setPersistentUserStyleSheetLocation(JSStringRef jsURL)
@@ -602,6 +662,32 @@ void LayoutTestController::setDatabaseQuota(unsigned long long quota)
     webkit_security_origin_set_web_database_quota(origin, quota);
 }
 
+JSValueRef LayoutTestController::originsWithLocalStorage(JSContextRef context)
+{
+    // FIXME: implement
+    return JSValueMakeUndefined(context);
+}
+
+void LayoutTestController::deleteAllLocalStorage()
+{
+        // FIXME: implement
+}
+
+void LayoutTestController::deleteLocalStorageForOrigin(JSStringRef originIdentifier)
+{
+        // FIXME: implement
+}
+
+void LayoutTestController::observeStorageTrackerNotifications(unsigned number)
+{
+        // FIXME: implement
+}
+
+void LayoutTestController::syncLocalStorage()
+{
+    // FIXME: implement
+}
+
 void LayoutTestController::setDomainRelaxationForbiddenForURLScheme(bool, JSStringRef)
 {
     // FIXME: implement
@@ -829,14 +915,20 @@ bool LayoutTestController::hasSpellingMarker(int from, int length)
     return DumpRenderTreeSupportGtk::webkitWebFrameSelectionHasSpellingMarker(mainFrame, from, length);
 }
 
-void LayoutTestController::dumpConfigurationForViewport(int availableWidth, int availableHeight)
+void LayoutTestController::dumpConfigurationForViewport(int deviceDPI, int deviceWidth, int deviceHeight, int availableWidth, int availableHeight)
 {
     WebKitWebView* webView = webkit_web_frame_get_web_view(mainFrame);
     ASSERT(webView);
-    DumpRenderTreeSupportGtk::dumpConfigurationForViewport(webView, availableWidth, availableHeight);
+    DumpRenderTreeSupportGtk::dumpConfigurationForViewport(webView, deviceDPI, deviceWidth, deviceHeight, availableWidth, availableHeight);
 }
 
 void LayoutTestController::setSerializeHTTPLoads(bool)
 {
     // FIXME: Implement if needed for https://bugs.webkit.org/show_bug.cgi?id=50758.
 }
+
+void LayoutTestController::setMinimumTimerInterval(double minimumTimerInterval)
+{
+    WebKitWebView* webView = webkit_web_frame_get_web_view(mainFrame);
+    DumpRenderTreeSupportGtk::setMinimumTimerInterval(webView, minimumTimerInterval);
+}
